package # Hide from PAUSE
  DBIx::Class::SQLMaker::SQLite;

use base qw( DBIx::Class::SQLMaker );
use Carp::Clan qw/^DBIx::Class|^SQL::Abstract/;

#
# SQLite does not understand SELECT ... FOR UPDATE
# Disable it here
sub _lock_select { '' };

1;
